-- config
local wsize = 18
local hsize = 55.6
local myname, _ = UnitName("player")
local _, myclass = UnitClass("player")
local statusbarcolor = {48/255, 48/255, 48/255}
local font = "Interface\\AddOns\\oUF_Led2\\media\\font.ttf"
local fontsize = 8

-- config end

MakeFS = function(parent, justify)
    local f = parent:CreateFontString(nil, "OVERLAY")
    f:SetFont("Interface\\AddOns\\oUF_Led2\\media\\font.ttf", 9, "THINOUTLINE", "MONOCHROME")
    f:SetShadowColor(0, 0, 0, 0)
	f:SetShadowOffset(0,0)
    if(justify) then f:SetJustifyH(justify) end
    return f
end

-- this part is taken from neav's raidframes <3

local function GetSpellName(spellID)
    local name = GetSpellInfo(spellID)
    return name
end

local function DeficitValue(self)
    if (self >= 1000) then
		return format('-%.1f', self/1000)
	else
		return self
	end
end

local dispellPriority = {
    ['Magic'] = 1,
    ['Poison'] = 4,
    ['Disease'] = 2,
    ['Curse'] = 3,
    ['None'] = 0,
}

local dispellFilter = {
    ['PRIEST'] = {
        ['Magic'] = true,
        ['Disease'] = true,
    },
    ['SHAMAN'] = {
        ['Poison'] = true,
        ['Disease'] = true,
        ['Curse'] = true,
    },
    ['PALADIN'] = {
        ['Poison'] = true,
    ['Magic'] = true,
        ['Disease'] = true,
    },
    ['MAGE'] = {
        ['Curse'] = true,
    },
    ['DRUID'] = {
        ['Curse'] = true,
        ['Poison'] = true,
    },
    ['DEATHKNIGHT'] = {},
    ['HUNTER'] = {},
    ['ROGUE'] = {},
    ['WARLOCK'] = {},
    ['WARRIOR'] = {},
}

local playerClass = select(2, UnitClass('player'))

local dispellClass = {}
if (dispellFilter[playerClass]) then
    for k, v in pairs(dispellFilter[playerClass]) do
        dispellClass[k] = v
    end
    dispellFilter = nil
end

local debuffList = setmetatable({
    [GetSpellName(6215)] = 3, -- fear
    
    [GetSpellName(67479)] = 10, -- impale
    [GetSpellName(66869)] = 8, -- burning bile
    
    [GetSpellName(67049)] = 9, -- incinerate flesh
    [GetSpellName(66823)] = 10, -- paralizing toxin
    [GetSpellName(72293)] = 1, -- Mark of the fallen champion

    
    [GetSpellName(24339)] = 9
},{ __index = function() 
    return 0 
end})


    --
    -- Copyright (c) 2009, Chris Bannister (oUF_Grid - zariel)
    -- All rights reserved.
    --
    
local function UpdateAura(self, event, unit)
	if (self.unit ~= unit) then 
        return 
    end
    
    local cur, tex, dis, co
    local debuffs = debuffList
    
	for i = 1, 40 do
		local name, _, buffTexture, count, dtype = UnitAura(unit, i, 'HARMFUL')
		if (not name) then 
            break 
        end

		if (not cur or (debuffs[name] >= debuffs[cur])) then
			if (debuffs[name] > 0 and debuffs[name] > debuffs[cur or 1]) then
				cur = name
				tex = buffTexture
				dis = dtype or 'none'
                co = count
			elseif (dtype and dtype ~= 'none') then
				if (not dis or (dispellPriority[dtype] > dispellPriority[dis])) then
					tex = buffTexture
					dis = dtype
                    co = count
				end
			end	
		end
	end

	if (dis) then
		if (dispellClass[dis] or cur) then
            self.Icon:Show()
            self.Icon.Icon:SetTexture(tex)
            
            local col = DebuffTypeColor[dis]
            self.Icon.Border:SetVertexColor(col.r, col.g, col.b)
    
            self.Icon.Count:SetText(co > 0 and co or '')
               
            self.Health.value:Hide()
            self.Name:Hide()
            
            self.Dispell = true
		elseif (self.Dispell) then
            self.Icon:Hide()
            
            self.Health.value:Show()
            self.Name:Show()
            
            self.Dispell = false
		end
	else
        self.Icon:Hide()
        
        self.Health.value:Show()
        self.Name:Show()
	end
end


  
-- VALUE FORMAT
  
local function do_format(v)
local string = ""
if v > 1000000 then
string = (floor((v/1000000)*10)/10).."m"
elseif v > 1000 then
string = (floor((v/1000)*10)/10)..""

else
string = v
end  
return string
end

local function HasDebuffType(unit, t)
for i=1,40 do
local name, _, _, _, debuffType = UnitDebuff(unit, i)
if not name then return
elseif debuffType == t then return true end
end
end
 
local function UpdateThreat(self, event, unit)
	if (self.unit ~= unit) then 
        return 
    end

 
 if (self.Aggro) then
        local threat = UnitThreatSituation(self.unit)
        if (threat == 3) then
            self.Aggro:SetText('|cFFFF0000*')
            
        else
            self.Aggro:SetText('')
              end
    end
end
  
local function updateHealth(self, event, unit, bar, min, max)
local tmpunitname

if unit then

tmpunitname = UnitName(unit)
local count = 4

if tmpunitname and tmpunitname:len() > count then

tmpunitname = tmpunitname:sub(1, count)
end
end

 
local c = max - min
local d = floor(min/max*100) 
local newmin = do_format(min)
local newmax = do_format(max)
local diff = do_format(max-min)
    
local color = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
if color then
	bar:SetStatusBarColor(color.r,color.g,color.b,1)
	bar.bg:SetVertexColor(color.r,color.g,color.b,0.45)
else
	bar.bg:SetVertexColor(1,0.3,0.3,1)
end

if UnitLevel(unit) == 0 then
	self.Name:SetTextColor(0.4,0.4,0.4)
	self.Name:SetText("")    
	bar.bg:SetVertexColor(0,0,0,0.4)
elseif UnitIsConnected(unit) ~= 1 then

self.Name:SetTextColor(0.4,0.4,0.4)
self.Name:SetText("off")    
bar.bg:SetVertexColor(0,0,0,0.4)

elseif UnitIsDeadOrGhost(unit) == 1 then

self.Name:SetTextColor(0.4,0.4,0.4)
self.Name:SetText("dead")    
bar.bg:SetVertexColor(0,0,0,0.4)

elseif d == 100 then

if color then
self.Name:SetTextColor(1,1,1,1)
end
self.Name:SetText(tmpunitname)

elseif min > 1 then

self.Name:SetTextColor(1,1,1)
self.Name:SetText("|cFFFF0000-|r"..diff)

else

self.Name:SetTextColor(0.4,0.4,0.4)
self.Name:SetText("..")    
bar.bg:SetVertexColor(0,0,0,0.4)
end
end
  
local stylefunc = function(self, unit)
  
self:SetScript("OnEnter", UnitFrame_OnEnter)
self:SetScript("OnLeave", UnitFrame_OnLeave)
self:SetAttribute('initial-height', wsize)
self:SetAttribute('initial-width', hsize)


local backdrop
self:SetBackdrop{
bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
insets = {left = 1, right = 1, top = 1, bottom = 1},
}
self:SetBackdropColor(0,0,0,1) 

self.Health = CreateFrame("StatusBar",nil,self)
self.Health:SetPoint("TOPLEFT",self,"TOPLEFT",2,-2)
self.Health:SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-2,2)
self.Health:SetStatusBarTexture("Interface\\AddOns\\oUF_Led2\\media\\HalM")
self.Health:SetOrientation("HORIZONTAL") 
self.Health:SetStatusBarColor(statusbarcolor[1],statusbarcolor[2],statusbarcolor[3])
self.Health.Smooth = true

self.Health.bg = self.Health:CreateTexture(nil, "BACKGROUND")
self.Health.bg:SetAllPoints(self.Health)
self.Health.bg:SetTexture("Interface\\AddOns\\oUF_Led2\\media\\HalM")
  
self.Name = self.Health:CreateFontString(nil, "ARTWORK")
self.Name:SetPoint("LEFT", 2, 0)
self.Name:SetPoint("RIGHT", -2, 0)
self.Name:SetFont(font,fontsize,"OUTLINEMONOCHROME")
self.Name:SetShadowColor(0,0,0,0)
self.Name:SetTextColor(1, 1, 1)

self.Aggro = self.Health:CreateFontString(nil, "OVERLAY")
self.Aggro:SetPoint("CENTER", 1, 10)
self.Aggro:SetFont(font,fontsize,"OUTLINEMONOCHROME")
self.Aggro:SetShadowColor(0,0,0,0)
self.Aggro:SetTextColor(1, 1, 1)

--[[ ReadyCheck support ]]--
self.ReadyCheck = self.Health:CreateTexture(nil, "OVERLAY")
self.ReadyCheck:SetPoint("CENTER", self.Health, 0, 0)
self.ReadyCheck:SetHeight(20)
self.ReadyCheck:SetWidth(20)

table.insert(self.__elements, UpdateThreat)
self:RegisterEvent('PLAYER_TARGET_CHANGED', UpdateThreat)
self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', UpdateThreat)
self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', UpdateThreat)

self.glossf = CreateFrame("Frame",nil,self.Health)
self.glossf:SetPoint("TOPLEFT",self.Health,"TOPLEFT",-2,2)
self.glossf:SetPoint("BOTTOMRIGHT",self.Health,"BOTTOMRIGHT",2,-2)
    
self.glosst = self.glossf:CreateTexture(nil, "ARTWORK")
self.glosst:SetAllPoints(self.glossf)
self.glosst:SetTexture(nil)
self.glosst:SetVertexColor(0,0,0)

self.Health.value = self.Health:CreateFontString(nil, 'OVERLAY')
self.Health.value:SetPoint('BOTTOM', 0, 5)
self.Health.value:SetShadowOffset(1, -1)

self.Icon = CreateFrame('Frame')
self.Icon:SetParent(self)
self.Icon:SetFrameStrata('HIGH')
    
self.Icon.Icon = self.Icon:CreateTexture(nil, 'ARTWORK', self.Icon)
self.Icon.Icon:SetPoint('CENTER', self.Health)
self.Icon.Icon:SetHeight(12)
self.Icon.Icon:SetWidth(12)
self.Icon.Icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)

self.Icon.Count = self.Icon:CreateFontString(nil, 'OVERLAY', self.Icon)
self.Icon.Count:SetPoint('BOTTOMRIGHT', self.Icon.Icon, 1, 0)
self.Icon.Count:SetFont(font, fontsize, 'OUTLINEMONOCHROME')
self.Icon.Count:SetShadowColor(0, 0, 0, 0)
self.Icon.Count:SetTextColor(1, 1, 1)
    
self.Icon.Border = self.Icon:CreateTexture(nil, 'BORDER', self.Icon)
self.Icon.Border:SetPoint('CENTER', self.Health)
self.Icon.Border:SetHeight(13)
self.Icon.Border:SetWidth(13)
self.Icon.Border:SetTexture(0,0,0)
self.Icon.Border:SetVertexColor(1, 1, 1)

self.Aggro = self.Health:CreateFontString(nil, "OVERLAY")
self.Aggro:SetPoint("BOTTOMRIGHT", -1, 1)
self.Aggro:SetFont(font,fontsize,"OUTLINEMONOCHROME")
self.Aggro:SetShadowColor(0,0,0,0)
self.Aggro:SetTextColor(1, 1, 1)

table.insert(self.__elements, UpdateThreat)
self:RegisterEvent('PLAYER_TARGET_CHANGED', UpdateThreat)
self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', UpdateThreat)
self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', UpdateThreat)

table.insert(self.__elements, UpdateAura)
self:RegisterEvent('UNIT_AURA', UpdateAura)
self:RegisterEvent('UNIT_DEAD', UpdateAura)


local ricon = self.Health:CreateTexture(nil, "OVERLAY")
ricon:SetHeight(10)
ricon:SetWidth(10)
ricon:SetPoint("BOTTOM", self.Name, "TOP", 0, -1)
ricon:SetTexture"Interface\\TargetingFrame\\UI-RaidTargetingIcons"
self.RaidIcon = ricon

if (not unit) then

self.Range = true
self.outsideRangeAlpha = 0.4
self.inRangeAlpha = 1
end
self.PostUpdateHealth = updateHealth
return self
end

local actstyle = "Led2Raid"
oUF:RegisterStyle(actstyle, stylefunc)
oUF:SetActiveStyle(actstyle)
 
local raid = {}
for i = 1, 5 do
table.insert(raid, oUF:Spawn("header", "oUF_Raid"..i))

if i == 1 then
raid[i]:SetPoint("BOTTOM", oUF_target, "TOP", 0, -72)
else
raid[i]:SetPoint("CENTER", raid[i-1], "CENTER", 0, -17)    
end

raid[i]:SetManyAttributes(
"showSolo", false,    
"showParty", false,
"showPlayer", true,  
"showRaid", true,   
"columnSpacing", -1,   
"unitsPerColumn", 1,   
"maxColumns", 5,   
"columnAnchorPoint", "LEFT",   
"xOffset", 3,  
"groupFilter", i )   

local raidToggle = CreateFrame("Frame")
raidToggle:RegisterEvent("PLAYER_LOGIN")
raidToggle:RegisterEvent("RAID_ROSTER_UPDATE")
raidToggle:RegisterEvent("PARTY_LEADER_CHANGED")
raidToggle:RegisterEvent("PARTY_MEMBERS_CHANGED")
raidToggle:SetScript("OnEvent", function(self)
		if InCombatLockdown() then
			self:RegisterEvent("PLAYER_REGEN_ENABLED")
		else
			self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		local numraid = GetNumRaidMembers()
		if numraid < 6 then
			raid[i]:Hide()
		else
			raid[i]:Show()
		end
	end
end)
end


