--[[-------------------------------------------------------------------------
  Trond A Ekseth grants anyone the right to use this work for any purpose,
  without any conditions, unless such conditions are required by law.
  ALZA uses this right to conquer the whole world!
  Ocularis twists the perspective of the sane world to make a new awesome layout
---------------------------------------------------------------------------]]

local settings = oUF_Led2_Settings["Solo&Party"]
local ColorGradient = oUF.ColorGradient
local _, class = UnitClass("player")
local format = string.format
local unpack = unpack
local _G = getfenv(0)
local BACKDROP = {
bgFile = [=[Interface\ChatFrame\ChatFrameBackground]=],
insets = {top = -1, bottom = -1, left = -1, right = -1}
}

oUF.Tags['[omgheistehleader]'] = function(unit)
	return UnitIsPartyLeader(unit) and '|cffffff00!|r '
end

oUF.Tags['[minus]'] = function(unit)
local perc = oUF.Tags['[perhp]'](unit)
return perc and perc < 100 and format('-')
end

oUF.Tags['ssstatus'] = function(unit)
return UnitIsDead(unit) and 'Dead' or UnitIsGhost(unit) and 'Ghost' or not UnitIsConnected(unit) and 'Offline'
end

oUF.Tags['[smarthpname]'] = function(u)
	return not oUF.Tags['ssstatus'](u) and oUF.Tags['[missinghp]'](u) or oUF.Tags['[name]'](u)
end

oUF.Tags['[smarthpnameparty]'] = function(u)
	return not oUF.Tags['ssstatus'](u) and oUF.Tags['[missinghp]'](u) or UnitName(u):sub(1, 7)
end

oUF.Tags['[smarthpnamefocus]'] = function(u)
	return not oUF.Tags['ssstatus'](u) and oUF.Tags['[missinghp]'](u) or UnitName(u):sub(1, 13)
end

oUF.Tags['[smartnamefocust]'] = function(u)
	return UnitName(u):sub(1, 4)
end



local MyUnits = { -- used for hiding cooldowns on target frame
    player = true,
    pet = true,
    vehicle = true,
}

local runeloadcolors = {
	[1] = {.69,.31,.31},
	[2] = {.69,.31,.31},
	[3] = {.33,.59,.33},
	[4] = {.33,.59,.33},
	[5] = {.31,.45,.63},
	[6] = {.31,.45,.63},
}

--[[ Font creation func ]]
MakeFS = function(parent, justify)
    local f = parent:CreateFontString(nil, "OVERLAY")
    f:SetFont(settings.font, settings.fsize, "THINOUTLINEMONOCHROME")
    f:SetShadowColor(0, 0, 0, 0)
	f:SetShadowOffset(0,0)
    if(justify) then f:SetJustifyH(justify) end
    return f
end

MakeFSB = function(parent, justify)
    local f = parent:CreateFontString(nil, "OVERLAY")
    f:SetFont(settings.font, settings.fsize * 2, "THINOUTLINEMONOCHROME")
    f:SetShadowColor(0, 0, 0, 0)
	f:SetShadowOffset(0,0)
    if(justify) then f:SetJustifyH(justify) end
    return f
end

--[[ Background creation func ]]
MakeBG = function(parent)
    local bg = parent:CreateTexture(nil, "BACKGROUND")
    bg:SetPoint("TOPLEFT", parent, -1, 1)
    bg:SetPoint("BOTTOMRIGHT", parent, 1, -1)
    bg:SetTexture(0, 0, 0)
end

--[[ Right click menu ]]
local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("(.)", string.upper, 1)

	if(unit=="party" or unit=="partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor")
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor")
	end
end

--[[ Short numbarz! ]]
local siValue = function(val)
    if val >= 10000000 then 
        return format("%.1fm", val / 1000000) 
    elseif val >= 1000000 then
        return format("%.2fm", val / 1000000) 
    elseif val >= 100000 then
        return format("%.0fk", val / 1000) 
    elseif val >= 10000 then
        return format("%.1fk", val / 1000) 
    else
        return val
    end
end

--[[ Health update ]]
local OverrideUpdateHealth = function(self, event, unit, bar, min, max)
    local r, g, b = 1, 1, 1

    if(max~=0) then
        r, g, b = ColorGradient(min/max, 1, 0, 0, 1, 1, 0, 0, 1, 0)
    end

    if(not UnitIsConnected(unit)) then
        bar:SetValue(0)
        bar.value:SetText("Off")
    elseif(UnitIsDead(unit)) then
        bar:SetValue(0)
        bar.value:SetText("Dead")
    elseif(UnitIsGhost(unit)) then
        bar:SetValue(0)
        bar.value:SetText("Ghost")
    else
        if(unit=="player") then
            bar.value:SetFormattedText(" |cff99CC66%s|r", siValue(min))
        elseif(unit=="pet") then
            bar.value:SetFormattedText(" |cff99CC66%s|r", siValue(min))
        elseif(unit=="target") then
            bar.value:SetFormattedText(" |cff99CC66%s|r", siValue(min))
		elseif(unit and unit:find("arena%d") and not unit:find("arena%dtarget")) then
			bar.value:SetFormattedText("|cffffffff%s|r |cffffffff-|r |cffffffff%d%%|r", siValue(min), floor(min / max * 100))
		elseif(unit and unit:find("arena%dtarget")) then
			bar.value:SetText()
        else
            bar.value:SetText()
        end
    end

    if(settings.ClassColor==true) then
        if(UnitIsPlayer(unit)) then
            local _, englass = UnitClass(unit)
            if(self.colors.class[englass]) then
                r, g, b = unpack(self.colors.class[englass])
            end
        else
            r, g, b = UnitSelectionColor(unit)
        end
    else
        r, g, b = unpack(settings.OwnColor)
    end
    
    if(UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) then
        r, g, b = .6, .6, .6
    end

    if(unit=="pet" and GetPetHappiness()) then
        r, g, b = unpack(self.colors.happiness[GetPetHappiness()])
    end
	
    bar:SetStatusBarColor(r, g, b)
    bar.bg:SetVertexColor(r, g, b, .2)
    if(self.Castbar) then
        self.Castbar:SetStatusBarColor(r, g, b)
        self.Castbar.bg:SetVertexColor(r, g, b, .2)
    end
end

--[[ Power update ]]
local powercolors = {
    ["MANA"] = {.2, .68, .78},
    ["RAGE"] = {.9, .1, .1},
    ["FUEL"] = {0, 0.55, 0.5},
    ["FOCUS"] = {.9, .9, .1},
    ["ENERGY"] = {.9, .9, .1},
    ["AMMOSLOT"] = {0.8, 0.6, 0},
    ["RUNIC_POWER"] = {.1, .9, .9},
    ["POWER_TYPE_STEAM"] = {0.55, 0.57, 0.61},
    ["POWER_TYPE_PYRITE"] = {0.60, 0.09, 0.17},
}

local PostUpdatePower = function(self, event, unit, bar, min, max)
    if(not UnitIsConnected(unit) or min==0) then
        bar.value:SetText()
    elseif(UnitIsDead(unit) or UnitIsGhost(unit)) then
        bar:SetValue(0)
        bar.value:SetText()
    elseif(unit=="player") then
        bar.value:SetFormattedText("|cff66FFFF%s|r", siValue(min)) 
	elseif (self:GetName():match"oUF_Arena") then
		bar.value:SetFormattedText("|cffffffff%s - %d%%|r", siValue(max - (max - min)), floor(min / max * 100))		
    else
        bar.value:SetText()
    end

    local r, g, b = 1, 1, 1
    if(settings.PowerColorByType==true) then
        local _, ptype = UnitPowerType(unit)
        if(powercolors[ptype]) then
            r, g, b = unpack(powercolors[ptype])
        end
    else
        r, g, b = unpack(settings.OwnPowerColor)
    end

    bar:SetStatusBarColor(r, g, b)
    bar.value:SetTextColor(r, g, b)
    bar.bg:SetVertexColor(r, g, b, .2)
end

--[[ Druid power update. Credits to p3lim for code and idea ]]
oUF.TagEvents["[druidpower]"] = "UNIT_MANA UPDATE_SHAPESHIFT_FORM"
oUF.Tags["[druidpower]"] = function(unit)
    local min, max = UnitPower(unit, 0), UnitPowerMax(unit, 0)
    return UnitPowerType(unit)~=0 and format("|cff0090ff%s|r -", tostring(siValue(min)))
end


--[[ Castbar styling ]]
local CustomTimeText = function(self, duration)
    self.Time:SetFormattedText("%.1f / %.1f", duration, self.max)
end

local sentTime = 0
local lagTime = 0

local UNIT_SPELLCAST_SENT = function (self, event, unit, spell, spellrank)
    if(self.unit~=unit) then return end
    sentTime = GetTime()
end

local PostCastStart = function(self, event, unit, name, rank, text, castid)
    lagTime = GetTime() - sentTime
end

local PostChannelStart = function(self, event, unit, name, rank, text)
    lagTime = GetTime() - sentTime
end

local OnCastbarUpdate = function(self, elapsed)
    if(self.casting) then
        local duration = self.duration + elapsed
        if(duration>=self.max) then
            self.casting = nil
            self:Hide()
            return
        end

        local width = self:GetWidth()
        local safeZonePercent = lagTime / self.max
        if(safeZonePercent>1) then safeZonePercent=1 end
        self.SafeZone:SetWidth(width * safeZonePercent)

        if(self.delay~=0) then
            self.Time:SetFormattedText("%.1f|cffff0000-%.1f|r", duration, self.delay)
        else
            self.Time:SetFormattedText("%.1f / %.1f", duration, self.max)
            self.Lag:SetFormattedText("%d ms", lagTime * 1000)
        end

        self.duration = duration
        self:SetValue(duration)

        self.Spark:SetPoint("CENTER", self, "LEFT", (duration / self.max) * self:GetWidth(), 0)

    elseif(self.channeling) then
        local duration = self.duration - elapsed

        if(duration<=0) then
            self.channeling = nil
            self:Hide()
            return
        end

        if(lagTime > 1e5) then
            lagTime = 0
            self.SafeZone:SetWidth(0)
        else
            local width = self:GetWidth()
            local safeZonePercent = lagTime / self.max
            if(safeZonePercent > 1 or safeZonePercent<=0) then safeZonePercent = 1 end
            self.SafeZone:SetWidth(width * safeZonePercent)
        end

        if(self.delay~=0) then
            self.Time:SetFormattedText("%.1f|cffff0000-%.1f|r", duration, self.delay)
        else
            self.Time:SetFormattedText("%.1f / %.1f", duration, self.max)
            self.Lag:SetFormattedText( "%d ms ", lagTime * 1000 )
        end

        self.duration = duration
        self:SetValue(duration)

        self.Spark:SetPoint("CENTER", self, "LEFT", (duration / self.max) * self:GetWidth(), 0)
    else
        self.unitName = nil
        self.channeling = nil
        self:SetValue(1)
        self:Hide()
    end
end

--[[ Aura settings ]]
local noHide = function(self)
    self:SetVertexColor(0, 0, 0)
end

if(not settings.Auras.DisablePlayerAuras) then
    _G["BuffFrame"]:Hide()
    _G["BuffFrame"]:UnregisterAllEvents()
    _G["BuffFrame"]:SetScript("OnUpdate", nil)
end

--[[ Temporary enchant stuff ]]
if(settings.Auras.StyleTempEnchants) then
    for i = 1, 2 do
        _G["TempEnchant"..i.."Icon"]:Hide()
        _G["TempEnchant"..i.."Count"]:Hide()
        _G["TempEnchant"..i.."Border"]:Hide()
        _G["TempEnchant"..i.."Duration"] = MakeFS(UIParent)
        _G["TempEnchant"..i]:ClearAllPoints()
        _G["TempEnchant"..i]:SetAllPoints(_G["TempEnchant"..i.."Duration"])
    end
elseif(not settings.Auras.DisablePlayerAuras) then
    _G["TemporaryEnchantFrame"]:Hide()
    _G["TemporaryEnchantFrame"]:SetScript("OnUpdate", nil)
    _G["TempEnchant1"]:SetScript("OnUpdate", nil)
    _G["TempEnchant2"]:SetScript("OnUpdate", nil)
end

local PostCreateAuraIcon = function(self, button, icons, index, isDebuff)
    button.cd:SetReverse()

    button.count:ClearAllPoints()
    button.count:SetPoint(unpack(settings.Auras.CountPoint))
    button.count:SetJustifyH("CENTER")
    button.count:SetFont(settings.font, settings.fsize, "OUTLINE")
    button.count:SetTextColor(unpack(settings.Auras.CountColor))

	button:SetBackdrop(BACKDROP)
	button:SetBackdropColor(0, 0, 0)

	button.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
	button.icon:SetDrawLayer('ARTWORK')
end

local PostUpdateAuraIcon = function(self, icons, unit, icon, index, offset, filter, isDebuff)
    if(icon.cd:IsShown() and isDebuff and not MyUnits[icon.owner] and not UnitIsPlayer(self.unit)) then
        icon.cd:Hide()
    end
end

--[[ Let's start! ]]
local func = function(self, unit)
    self.menu = menu
    self:SetScript("OnEnter", UnitFrame_OnEnter)
    self:SetScript("OnLeave", UnitFrame_OnLeave)
    self:RegisterForClicks("anyup")
    self:SetAttribute("*type2", "menu")
    self.disallowVehicleSwap = true
    MakeBG(self)
	
	--[[Rclick arena focus?]]
	
	if (unit and unit:find("arena%d")) then
		self:SetAttribute("type2", "focus")
	end
	
        
    local s = settings.Frames[unit] or settings.Frames.party

    --[[ Frame sizes ]]
	if(unit and unit:find("arena%d") and not unit:find("arena%dtarget")) then
		self:SetAttribute("initial-height", 10)
		self:SetAttribute("initial-width", 150)
	elseif(unit and unit:find("arena%dtarget")) then
		self:SetAttribute("initial-height", 10)
		self:SetAttribute("initial-width", 30)
	else
	self:SetAttribute("initial-width", s.Width)
	self:SetAttribute("initial-height", s.Height)
	end
	
    --[[ Healthbar ]]
    local hp = CreateFrame("StatusBar", nil, self)
    hp:SetStatusBarTexture(settings.texture)
    hp.frequentUpdates = true
	hp:SetPoint("TOPRIGHT")
    hp:SetPoint("TOPLEFT")
    hp:SetHeight(s.Height)
	--hp:SetHeight(s.ManaBarHeight==0 and s.Height or (s.Height - s.ManaBarHeight - 1))

    local hpbg = hp:CreateTexture(nil, "BACKGROUND")
    hpbg:SetTexture(settings.texture)
    hpbg:SetAllPoints(hp)

    local hpp = MakeFSB(hp)
    if(unit=="player" or (unit and unit:find("arena%d"))) then
        hpp:SetPoint("LEFT", hp, "LEFT", 0, 3)
    elseif(unit=="target" or unit=="pet") then
        hpp:SetPoint("LEFT", hp, "LEFT", 2, 3)
    end

    hp.bg = hpbg
    hp.value = hpp
    self.Health = hp
    self.OverrideUpdateHealth = OverrideUpdateHealth

    --[[ Manabar ]]
    local pp = CreateFrame("StatusBar", nil, self)
    pp:SetStatusBarTexture(settings.texture)
    pp.frequentUpdates = true
	pp:SetPoint("BOTTOMLEFT", 0, -6)
    pp:SetPoint("BOTTOMRIGHT", 0, -6)
	--pp:SetPoint("BOTTOMLEFT")
    --pp:SetPoint("BOTTOMRIGHT")
	if(unit and unit:find("arena%d") and not unit:find("arena%dtarget")) then
    pp:SetHeight(2)
	else
    pp:SetHeight(s.ManaBarHeight)
	end
	MakeBG(pp)

    local ppbg = pp:CreateTexture(nil, "BACKGROUND")
    ppbg:SetTexture(settings.texture)
    ppbg:SetAllPoints(pp)

    local ppp = MakeFS(pp)
    if(unit=="player") then
        ppp:SetPoint("RIGHT", hp, "RIGHT", -2, 6)
    elseif(unit=="pet") then
        ppp:SetPoint("BOTTOMRIGHT", hp, "TOPRIGHT", 2, 3)
    end

    pp.bg = ppbg
    pp.value = ppp
    self.Power = pp
    self.PostUpdatePower = PostUpdatePower

    --[[ Info text ]]
    local info = MakeFS(hp)
	local info2 = MakeFS(hp)
    if(unit=="target") then
        info:SetPoint("TOPRIGHT", hp, "TOPRIGHT", 0, 5)
        info:SetJustifyH("RIGHT")
        info:SetPoint("RIGHT")
        self:Tag(info, "[difficulty][plus]|r [name] ")
    elseif(unit=="targettarget") then
        info:SetPoint("TOPRIGHT", hp, "BOTTOMRIGHT", -3, 12)
        info:SetPoint("RIGHT")
        self:Tag(info, "[name]")
	elseif(unit=="focus") then
		info:ClearAllPoints()
        info:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, 4)
		info.frequentUpdates = true
		self:Tag(info, "|cff00EEFF[minus]|r[smarthpnamefocus]")
	elseif(unit=="focustarget") then 
		info:ClearAllPoints()
        info:SetPoint("TOPRIGHT", self, "TOPRIGHT", 0, 4)
        self:Tag(info, "[smartnamefocust]")
		--info:SetWidth(31)
		--info:SetHeight(7)
		info:SetJustifyH("RIGHT")
    elseif(not unit) then
        info:SetParent(self.Health)
        info:SetPoint("RIGHT", self.Health, -2, 5)
        info:SetJustifyH("CENTER")
		info.frequentUpdates = true
        self:Tag(info, "|cff00EEFF[omgheistehleader][minus]|r[smarthpnameparty]")
    end

	
    --[[ Auras! ]]
	
	if(unit=="player" and not settings.Auras.DisablePlayerAuras) then
        local buffs = CreateFrame("Frame", nill, self)
        buffs:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -5, -25)
        buffs.initialAnchor = "TOPRIGHT"
        buffs["growth-y"] = "DOWN"
        buffs["growth-x"] = "LEFT"
        buffs:SetHeight(500)
        buffs:SetWidth((settings.Auras.PlayerBuffSize + settings.Auras.Spacing) * 20)
        buffs.spacing = settings.Auras.PlayerSpacing
        buffs.size = settings.Auras.PlayerBuffSize
        self.Buffs = buffs
		self.Buffs.num = 36

        local debuffs = CreateFrame("Frame", nill, self)
        debuffs:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -5, -105)
        debuffs.initialAnchor = "TOPRIGHT"
        debuffs["growth-x"] = "LEFT"
        debuffs["growth-y"] = "DOWN"
        debuffs:SetHeight(500)
        debuffs:SetWidth((settings.Auras.PlayerDeBuffSize + settings.Auras.Spacing) * 20)
        debuffs.spacing = settings.Auras.PlayerSpacing
        debuffs.size = settings.Auras.PlayerDeBuffSize
        self.Debuffs = debuffs
		self.Debuffs.num = 36

       
    end

    if(unit=="target") then
        local auras = CreateFrame("Frame", nill, self)
        auras:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 17)
        auras.initialAnchor = "BOTTOMLEFT"
        auras["growth-x"] = "RIGHT"
        auras["growth-y"] = "UP"
        auras.numDebuffs = 22
        auras.numBuffs = 22
        auras:SetHeight(500)
        auras:SetWidth(settings.Frames.target.Width + 2)
        auras.spacing = settings.Auras.Spacing
        auras.size = settings.Auras.Size
        auras.gap = true
		auras:SetScale(1)
        --auras.onlyShowPlayer = true
        self.Auras = auras
    end
	
	if(unit and unit:find("arena%d") and not unit:find("arena%dtarget")) then
		self.Debuffs = CreateFrame("Frame", nil, self)
		self.Debuffs.num = 5
		self.Debuffs.size = 24
		if(IsAddOnLoaded("oUF_Trinkets")) or (IsAddOnLoaded("oUF_PvPTrinket")) then
			self.Debuffs:SetPoint("TOPLEFT", self, "TOPRIGHT", 30, 0.5)
		else
			self.Debuffs:SetPoint("TOPLEFT", self, "TOPRIGHT", 3, 0.5)
		end
		self.Debuffs.initialAnchor = "TOPLEFT"
        self.Debuffs["growth-x"] = "RIGHT"
        self.Debuffs["growth-y"] = "DOWN"
		self.Debuffs:SetHeight(24)
        self.Debuffs:SetWidth(125)
		self.Debuffs.spacing = 1.5
	end
	
	--[[pvp trink]]
	if (unit and unit:find("arena%d") and (not unit:find("arena%dtarget")) and (not unit:find("arena%darena%dpet"))) then
		self.Trinket = CreateFrame("Frame", nil, self)
		self.Trinket:SetHeight(16)
		self.Trinket:SetWidth(16)
		self.Trinket:SetPoint("TOPLEFT", self, "TOPRIGHT", 5, 0)
		self.Trinket.trinketUseAnnounce = true
		self.Trinket.trinketUpAnnounce = true
		
		local trborder = self.Trinket:CreateTexture(nil,"BORDER")
		trborder:SetTexture(0,0,0,1)
		trborder:SetVertexColor(0.37, 0.3, 0.3)
		trborder:SetWidth(18)
        trborder:SetHeight(18)
		trborder:SetPoint("CENTER", self.Trinket)
		--trborder:SetPoint("TOPLEFT", self.Trinket, -1, 1)
		--trborder:SetPoint("BOTTOMRIGHT", self.Trinket, 1, -1)
		
		self.PvPTrinket = self:CreateTexture(nil, "OVERLAY")
		self.PvPTrinket:SetPoint("TOPLEFT", self, "TOPRIGHT", 3, 1)
		self.PvPTrinket:SetHeight(16)
		self.PvPTrinket:SetWidth(16)
	end

    if(settings.Auras.SpawnFocusDebuffs and unit=="focus") then
        local debuffs = CreateFrame("Frame", nill, self)
        debuffs:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -4)
        debuffs.initialAnchor = "TOPLEFT"
        debuffs["growth-x"] = "RIGHT"
        debuffs["growth-y"] = "DOWN"
        debuffs:SetHeight(20)
        debuffs:SetWidth(104)
        debuffs.spacing = 1
        debuffs.size = 14
        debuffs.num = 5
        self.Debuffs = debuffs
    end
	
	if(settings.Auras.SpawnPartyDebuffs and not unit) then
        local debuffs = CreateFrame("Frame", nill, self)
        debuffs:SetPoint("CENTER", self, "TOPLEFT", 52, -31)
        debuffs.initialAnchor = "TOPLEFT"
        debuffs["growth-x"] = "RIGHT"
        debuffs["growth-y"] = "DOWN"
        debuffs:SetHeight(20)
        debuffs:SetWidth(104)
        debuffs.spacing = 2
        debuffs.size = 13
        debuffs.num = 3
        self.Debuffs = debuffs
    end

    self.PostCreateAuraIcon = PostCreateAuraIcon
    if(settings.Auras.HideCooldownForDebuffs==true and unit=="target") then self.PostUpdateAuraIcon = PostUpdateAuraIcon end
	
--[[ Castbar <3 Faith ]]

	if (unit == 'player' or unit == 'target') then 
	    self.Castbar = CreateFrame('StatusBar', nil, self) 
	    self.Castbar:SetStatusBarTexture(settings.texture) 
	    self.Castbar:SetStatusBarColor(0.5,0.5,0.5) 
	 
	    self.Castbar.Text = self.Castbar:CreateFontString(nil, 'OVERLAY') 
		if (unit=='player') then
			self.Castbar.Text:SetPoint('LEFT', self.Castbar, 2, 1) 
		end
	    self.Castbar.Text:SetShadowOffset(1, -1) 
	    self.Castbar.Text:SetTextColor(1, 1, 1) 
	    self.Castbar.Text:SetJustifyH('LEFT') 
	    self.Castbar.Text:SetHeight(10) 
	    self.Castbar.Text:SetWidth(200) 
	 
	    self.Castbar:SetFrameStrata("LOW") 
	 
		if(unit == "player") then 
			self.Castbar:SetStatusBarColor(r, g, b) 
			self.Castbar:SetBackdrop({ 
				bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16, 
				insets = {top = -1, left = -1, bottom = -1, right = -1,5}}) 
			self.Castbar:SetBackdropColor(0, 0, 0) 
	 
			--[[self.Castbar.Icon = self.Castbar:CreateTexture(nil, "ARTWORK") 
			self.Castbar.Icon:SetPoint("LEFT", self.Castbar, -20, 0) 
			self.Castbar.Icon:SetTexCoord(0.1,0.9,0.1,0.9) ]]--
	 
			self.Castbar:SetPoint('CENTER', "oUF_player", 'CENTER', 0, 20) 
			self.Castbar:SetWidth(270) 
			self.Castbar:SetHeight(4) 
			self.Castbar.Text:SetFont(settings.font, settings.fsize, "THINOUTLINE") 
			self.Castbar.Text:Hide()
			--[[self.Castbar.Icon:SetHeight(19) 
			self.Castbar.Icon:SetWidth(19) ]]--
	        
			self.Castbar.SafeZone = self.Castbar:CreateTexture(nil, 'BORDER')
			self.Castbar.SafeZone:SetTexture(texture)
			self.Castbar.SafeZone:SetVertexColor(0.5, 0.5, 0.5, 0.65)
          	        
			self.Castbar.Time = self.Castbar:CreateFontString(nil, 'OVERLAY') 
			self.Castbar.Time:SetPoint('RIGHT', self.Castbar, -1, 1) 
			self.Castbar.Time:SetFont(settings.font, settings.fsize, "THINOUTLINE") 
			self.Castbar.Time:SetTextColor(1, 1, 1)
			self.Castbar.Time:SetJustifyH('RIGHT') 
			self.Castbar.Time:Hide()
		else 
			self.Castbar:SetStatusBarColor(1, 0, 0) 
			self.Castbar:SetBackdrop({ 
				bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16, 
				insets = {top = -1, left = -1, bottom = -1, right = -21}}) 
			self.Castbar:SetBackdropColor(0, 0, 0) 
	 
			self.Castbar.Icon = self.Castbar:CreateTexture(nil, "ARTWORK") 
			self.Castbar.Icon:SetPoint("RIGHT", self.Castbar, 20, 0) 
			self.Castbar.Icon:SetTexCoord(0.1,0.9,0.1,0.9)
			
			self.Castbar:SetPoint('CENTER', "oUF_player", 'CENTER', -10, 35) 
			self.Castbar:SetWidth(250) 
			self.Castbar:SetHeight(15) 
			self.Castbar.Text:SetFont(settings.font, settings.fsize, "THINOUTLINE") 
			self.Castbar.Text:SetPoint('LEFT', self.Castbar, 1, -2) 
			self.Castbar.Icon:SetHeight(15) 
			self.Castbar.Icon:SetWidth(15)
	        
			self.Castbar.Time = self.Castbar:CreateFontString(nil, 'OVERLAY') 
			self.Castbar.Time:SetPoint('RIGHT', self.Castbar, -1, 1) 
			self.Castbar.Time:SetFont(settings.font, settings.fsize, "THINOUTLINE") 
			self.Castbar.Time:SetTextColor(1, 1, 1) 
			self.Castbar.Time:SetJustifyH('RIGHT') 
			self.Castbar.Time:Hide()
		end	 
		self.Castbar.bg = self.Castbar:CreateTexture(nil, 'BORDER') 
		self.Castbar.bg:SetAllPoints(self.Castbar) 
		self.Castbar.bg:SetTexture(0, 0, 0) 
	end
	
    --[[ Runebar ]]
    if(settings.Plugins.RuneBarPlugin and unit=="player" and class=="DEATHKNIGHT" and IsAddOnLoaded("oUF_RuneBar")) then
        local runebar = MakeFS(self)
        runebar:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -2, 3)
        self.RuneBar = runebar
    end

    --[[ Druid mana ]]
    if(unit=="player" and class=="DRUID") then
        local druid = MakeFS(hp)
        druid:SetPoint("RIGHT", ppp, "LEFT", -1, 0)

        self:Tag(druid, "[druidpower]")
        self.DruidPower = druid
    end

	--[[ Runes ]]
	if(settings.RuneBar and class=="DEATHKNIGHT" and unit=="player") then
		self.Runes = CreateFrame('Frame', nil, self)
		self.Runes:SetPoint('BOTTOM', UIParent, 'BOTTOM', 0, 225)
		self.Runes:SetHeight(4)
		self.Runes:SetWidth(s.Width)
		self.Runes.anchor = "TOPLEFT"
		self.Runes.growth = "RIGHT"
		self.Runes.height = 3
		self.Runes.width = ((s.Width -5) / 6)
		self.Runes.spacing = 1

		MakeBG(self.Runes)
		

		for i = 1, 6 do
			self.Runes[i] = CreateFrame("StatusBar", self:GetName().."_Runes"..i, self.Runes)
			self.Runes[i]:SetStatusBarTexture(settings.texture)
			self.Runes[i]:SetStatusBarColor(unpack(runeloadcolors[i]))
		end
	end
	
	if not(settings.RuneBar) and(settings.HideDefaultRunes) then 
		RuneFrame:Hide()
	end
	
    --[[ oUF_Reputation support ]]
    if(settings.Plugins.oUF_Reputation == "Solid" and unit=="player" and IsAddOnLoaded("oUF_Reputation") and UnitLevel('player') == MAX_PLAYER_LEVEL) then
        local rep = CreateFrame("StatusBar", nil, self)
        rep:SetStatusBarTexture(settings.texture)
		rep:SetPoint('TOPLEFT', UIParent, 'BOTTOMLEFT', 0, 19)
		rep:SetPoint('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', 0, 18)
        rep:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3])
            
        rep.Tooltip = true
        self.Reputation = rep
		
	elseif(settings.Plugins.oUF_Reputation == "MouseOver" and unit=="player" and IsAddOnLoaded("oUF_Reputation") and UnitLevel('player') == MAX_PLAYER_LEVEL) then 
	    local rep = CreateFrame("StatusBar", nil, self)
        rep:SetStatusBarTexture(settings.texture)
		rep:SetPoint('TOPLEFT', UIParent, 'BOTTOMLEFT', 1, 19)
		rep:SetPoint('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', -1, 1)
        rep:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3])
            
        local rtext = MakeFS(rep)
        rtext:SetPoint("CENTER")

        rep.Tooltip = true
        rep.Text = rtext
        self.Reputation = rep
		
		rep:RegisterEvent("PLAYER_ENTERING_WORLD")
		rep:SetScript("OnEvent", function(self)
					self:SetAlpha(0)
		end)
				
		rep:SetScript ('OnEnter', function(self) self:SetAlpha(1) end)
		rep:SetScript ('OnLeave', function(self) self:SetAlpha(0) end)	
    end
        
    --[[ oUF_Swing support ]]
    if(settings.Plugins.oUF_Swing and unit=="player" and IsAddOnLoaded("oUF_Swing")) then
        local sw = CreateFrame("StatusBar", nil, self)
        sw:SetStatusBarTexture(settings.texture)
        sw:SetHeight(4)
        sw:SetWidth(s.Width)
        sw:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 18)
        sw:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3])
            
        local stext = MakeFS(sw)
        stext:SetPoint("CENTER")

        MakeBG(sw)
        sw.Text = stext
        self.Swing = sw
    end


    --[[ oUF_BarFader support ]]
    if(settings.Plugins.oUF_BarFader and unit=="player" and IsAddOnLoaded("oUF_BarFader")) then
        self.BarFade = true
        self.BarFaderMinAlpha = 0
        self.BarFaderMaxAlpha = 1
    end

    --[[ oUF_DebuffHighlight support ]]
    if(settings.Plugins.oUF_DebuffHighlight and IsAddOnLoaded("oUF_DebuffHighlight")) then
        local dh = self.Health:CreateTexture(nil, "OVERLAY")
        dh:SetAllPoints(self.Health)
        dh:SetTexture(settings.debufftex)
        dh:SetVertexColor(0, 0, 0, 0)
        dh:SetBlendMode("ADD")

        self.DebuffHighlight = dh
        self.DebuffHighlightAlpha = 1
        self.DebuffHighlightFilter = false
    end

    --[[ Raid Target Icon ]]
    if(settings.Icons.RaidTarget and (unit=="player" or unit=="target")) then
        local ricon = self.Health:CreateTexture(nil, "OVERLAY")
        ricon:SetHeight(16)
        ricon:SetWidth(16)
        ricon:SetPoint("CENTER")
        self.RaidIcon = ricon
    end

    --[[ Resting icon ]]
    if(settings.Icons.Resting and unit=="player") then
        local rest = self.Health:CreateTexture(nil, "OVERLAY")
        rest:SetHeight(20)
        rest:SetWidth(20)
        rest:SetPoint("LEFT")
        self.Resting = rest
    end

    --[[ Combat icon ]]
    if(settings.Icons.Combat and unit=="player") then
        local combat = self.Health:CreateTexture(nil, "OVERLAY")
        combat:SetHeight(18)
        combat:SetWidth(18)
        combat:SetPoint("LEFT")
        self.Combat = combat
    end
        
    --[[ PvP icon ]]
    if(settings.Icons.PvP and (unit=="player" or unit=="target")) then
        local pvp = self.Health:CreateTexture(nil, "OVERLAY")
        pvp:SetHeight(24)
        pvp:SetWidth(24)
        if(unit=="player") then
            pvp:SetPoint("TOPLEFT", self, "TOPLEFT", -7, 8)
        else
            pvp:SetPoint("TOPRIGHT", self, "TOPRIGHT", 17, 8)
        end
        self.PvP = pvp
    end
	
	--[[ Xp Bar ]]
	
	if(settings.Plugins.oUF_Experience == "Solid" and unit == "player" and not (UnitLevel("player") == MAX_UNIT_LEVEL) and IsAddOnLoaded("oUF_Experience")) then
				
				self.Experience = CreateFrame('StatusBar', nil, self)
				self.Experience:SetPoint('TOPLEFT', UIParent, 'BOTTOMLEFT', 0, 19)
				self.Experience:SetPoint('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', 0, 18)
				self.Experience:SetStatusBarTexture(settings.texture)
				self.Experience:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3])
				self.Experience.Tooltip = true

				self.Experience.Rested = CreateFrame('StatusBar', nil, self)
				self.Experience.Rested:SetAllPoints(self.Experience)
				self.Experience.Rested:SetStatusBarTexture(settings.texture)
				self.Experience.Rested:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3], 0.3)
				
	elseif(settings.Plugins.oUF_Experience == "MouseOver" and unit == "player" and not (UnitLevel("player") == MAX_UNIT_LEVEL) and IsAddOnLoaded("oUF_Experience")) then
				
				self.Experience = CreateFrame('StatusBar', nil, self)
				self.Experience:SetPoint('TOPLEFT', UIParent, 'BOTTOMLEFT', 1, 19)
				self.Experience:SetPoint('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', -1, 1)
				self.Experience:SetStatusBarTexture(settings.texture)
				self.Experience:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3])
				self.Experience.Tooltip = true

				self.Experience.Rested = CreateFrame('StatusBar', nil, self.Experience)
				self.Experience.Rested:SetAllPoints(self.Experience)
				self.Experience.Rested:SetStatusBarTexture(settings.texture)
				self.Experience.Rested:SetStatusBarColor(self.colors.class[class][1], self.colors.class[class][2], self.colors.class[class][3], 0.3)
				
				self.Experience:RegisterEvent("PLAYER_ENTERING_WORLD")
				self.Experience:SetScript("OnEvent", function(self)
					self:SetAlpha(0)
				end)
				
				self.Experience:SetScript ('OnEnter', function(self) self:SetAlpha(1) end)
				self.Experience:SetScript ('OnLeave', function(self) self:SetAlpha(0) end)
	end

    --[[ Combo Points ]]
    if(unit=="target") then
        local cp = MakeFSB(self.Health, "RIGHT")
        cp:SetPoint("RIGHT", self.Health, "LEFT", -5, -3)
        self.CPoints = cp
    end

    --[[ Fading for party ]]
    if(not unit) then
        self.Range = true 
        self.inRangeAlpha = 1.0 
        self.outsideRangeAlpha = 0.5
    end
    
    --[[ Speed up updating ToT ]]
    if(unit=="target") then
        self:RegisterEvent("UNIT_TARGET", function(self, event, unit)
            if(unit=="target") then
                oUF.units.targettarget:PLAYER_ENTERING_WORLD'OnTargetUpdate'
            end
        end)
    end

    return self
end

oUF:RegisterStyle("Led2", func)
oUF:SetActiveStyle("Led2")

--[[ Positions ]]
oUF:Spawn("player", "oUF_player"):SetPoint(unpack(settings.Positions.player))
oUF:Spawn("target", "oUF_target"):SetPoint(unpack(settings.Positions.target))
oUF:Spawn("targettarget", "oUF_targettarget"):SetPoint(unpack(settings.Positions.targettarget))
oUF:Spawn("focus", "oUF_focus"):SetPoint(unpack(settings.Positions.focus))
oUF:Spawn("focustarget", "oUF_focustarget"):SetPoint(unpack(settings.Positions.focustarget))
oUF:Spawn("pet", "oUF_pet"):SetPoint(unpack(settings.Positions.pet))

local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint(unpack(settings.Positions.party))
party:SetManyAttributes("showParty", true, "columnSpacing", 7, "showPlayer", false, 'unitsPerColumn', 1, 'yOffset', -15, 'columnAnchorPoint', "LEFT", "maxColumns", 4)
--party:SetManyAttributes("showParty", true, "yOffset", -5)
party:Show()

local arena = {}
for i = 1, 5 do
	arena[i] = oUF:Spawn("arena"..i, "oUF_Arena"..i)
	if i == 1 then
		arena[i]:SetPoint("BOTTOM", UIParent, "BOTTOM", -432, 266)
	else
		arena[i]:SetPoint("BOTTOM", arena[i-1], "TOP", 0, 25)
	end
end
	
local arenatarget = {}
for i = 1, 5 do
	arenatarget[i] = oUF:Spawn("arena"..i.."target", "oUF_Arena"..i.."target")
	if i == 1 then
		arenatarget[i]:SetPoint("TOPLEFT", arena[i], "TOPLEFT", -35, 0)
	else
		arenatarget[i]:SetPoint("BOTTOM", arenatarget[i-1], "TOP", 0, 25)
	end
end

--[[ Hide party when joining a raid ]]

if(settings.HidePartyInRaid) then
    local partyToggle = CreateFrame("Frame")
    partyToggle:RegisterEvent("PLAYER_LOGIN")
    partyToggle:RegisterEvent("RAID_ROSTER_UPDATE")
    partyToggle:RegisterEvent("PARTY_LEADER_CHANGED")
    partyToggle:RegisterEvent("PARTY_MEMBERS_CHANGED")
    partyToggle:SetScript("OnEvent", function(self)
        if(InCombatLockdown()) then
            self:RegisterEvent("PLAYER_REGEN_ENABLED")
        else
            self:UnregisterEvent("PLAYER_REGEN_ENABLED")
			local numraid = GetNumRaidMembers()
			if numraid > 0 and (numraid > 5 or numraid ~= GetNumPartyMembers() + 1) then
                party:Hide()
            else
                party:Show()
            end
        end
    end)
end

if(settings.ViewPortFrames) then
local botframe = CreateFrame("Frame",nil,UIParent)
botframe:SetFrameStrata("BACKGROUND")
botframe:SetWidth(20000) -- Set these to whatever height/width is needed 
botframe:SetHeight(20) -- for your Texture

local bottex = botframe:CreateTexture(nil,"BACKGROUND")
bottex:SetTexture(0, 0, 0, 0.6)
bottex:SetAllPoints(botframe)
botframe.texture = bottex

botframe:SetPoint("BOTTOM", UIParent, "BOTTOM" ,0, 0)
botframe:Show()

local topframe = CreateFrame("Frame",nil,UIParent)
topframe:SetFrameStrata("BACKGROUND")
topframe:SetWidth(20000) -- Set these to whatever height/width is needed 
topframe:SetHeight(20) -- for your Texture

local toptex = topframe:CreateTexture(nil,"BACKGROUND")
toptex:SetTexture(0, 0, 0, 0.6)
toptex:SetAllPoints(topframe)
topframe.texture = toptex

topframe:SetPoint("TOP", UIParent, "TOP" ,0, 0)
topframe:Show()
end 

local _, _, _, Senabled, _, Sreason, _ = GetAddOnInfo("Skada")
if Senabled == 1 then
local widget = CreateFrame("Button", "Skada Hider", UIParent);
local wdtex = widget:CreateTexture(nil,"BACKGROUND")
wdtex:SetTexture(0,0,0,0)
wdtex:SetAllPoints(widget)
widget:SetWidth(220); widget:SetHeight(20); widget:SetPoint("BOTTOMRIGHT");
widget:RegisterForClicks("AnyUp");
widget:SetScript("OnClick", function (self, button, up)
local S = getglobal("SkadaBarWindowSkada")
    if S ~= nil then end
    if S:IsVisible() then
    Skada:ToggleWindow()
	UIFrameFadeIn(PallyPowerFrame, 0.2, 0, 1)
    else
    Skada:ToggleWindow()
	UIFrameFadeIn(SkadaBarWindowSkada, 0.2, 0, 1)
	UIFrameFadeOut(PallyPowerFrame, 0.2, 1, 0)
    end
end);
widget:RegisterEvent("PLAYER_ENTERING_WORLD")
		widget:SetScript("OnEvent", function(self)
		local S = getglobal("SkadaBarWindowSkada")
		if S ~= nil then end
		if S:IsVisible() then
			UIFrameFadeOut(PallyPowerFrame, 0.2, 1, 0)
		else end
end)
else end

--[[ testmode ]]
local testui = TestUI or function() end
TestUI = function()
testui()
UnitAura = function()
-- name, rank, texture, count, dtype, duration, timeLeft, caster
return 'Durka Durk', 'Rank 2', 'Interface\\Icons\\Spell_Holy_Penance', random(5), 'Magic', 0, 0, "player"
end
if(oUF) then
for i, v in pairs(oUF.units) do
if(v.UNIT_AURA) then
v:UNIT_AURA("UNIT_AURA", v.unit)
end
end
end
end
SlashCmdList.TestUI = TestUI
SLASH_TestUI1 = "/testui"